由于**条件查询、分页显示**是非常重要的基本管理工具，所以jxWebUI内置了一个数据表的条件查询、分页显示工作流及相应组件来简化其实现。

此功能包括三个部分：表设计、数据库访问、将数据查询页面和数据源进行关联。

## 数据表设计

一个完整的**条件查询、分页显示**数据表包括三个部分：

- 查询条件
- 数据表
- 分页控件

其中分页控件在jxWebUI中已经和数据表整合在了一起，只需要在定义数据表时指定分页控件相关的参数即可。

由于jxWebUI中为了美观，几乎所有控件都使用表格来进行行列对齐。所以实际上是由一个查询条件表和一个结果数据表组合而成的。

查询条件表可以根据查询条件来任意设计，但必须要有一个按钮，一般命名为查询，其必须关联cmd的事件函数reSearch。

数据表则必须包含如下的属性：

- bind=tableTotalCount，用于jxWebUI在用户点击了查询按钮后，根据查询出来的数据总数来初始化分页控件的总条数
- pagination=true，启用分页控件，带有分页查询功能的数据表，主要用于输出数据，一般不应用于输入数据

<font color=red size=3>注1：</font>数据表如果不定义pagination=true，就是一个纯粹用来输入输出表数据的录入表，就需要开发者自行读取与输出数据了

<font color=red size=3>注2：</font>数据表的数据是整表读取【capaInstance.getInput(表名)】、整表输出【capaInstance.set_output_datatable(表名,行数据数组)】

<font color=red size=3>注3：</font>录入表在需要录入时应定义newRow=true属性，这时就可以数据表右上角出现一个白色的加号，点击该加号就可以添加新行供用户录入数据

整体的定义形如：
	
	@capa.page
	def test_datatable(ci, db, ctx):
	'''
		table table1 title='查询条件',width=900:
			row
				text text1 text='设备类型',bind=text1,width=200
				input input1 bind=devType,width=200
			row
				text text2 text='设备名',width=200
				input input2 bind=devName,width=200
			row
				button button1 text='查询',width=100,motion=cmd,demand=reSearch
		;
		dataTable table2 title='查询结果',bind=tableTotalCount,width=900,pagination=true:
			col devID head devID hide=true, width=200
			col devType head 设备类型 width=200
			col devName head 设备名称 width=200
			col op head 操作 type a width=200,capaname='test.query_device',motion=cmd,
				demand=active1,text='查看数据',ignoreCapaid=true,primary=true,
				require=[{'paramName':'devType'},{'paramName':'devName'}]
		;

<font color=red size=3>注：</font>分页查询、分页显示的数据表有一个内置的工作流，为了使得这个工作流能自动化的工作，jxWebUI占用了两个固定的cmd事件：search、reSearch。请不要将自己的任何事件命名为这两个名字

显示效果如下：

![test_web](http://115.29.52.95:10018/images/dt_1.png)


## 数据库访问

分页查询、分页显示的数据表会自动执行数据库的查询，为了实现此功能，我们需要完成三个步骤：

1、设置数据库连接获取

首先需要定义你自己程序的数据库连接获取函数。如mysql数据库可如下定义：

	import pymysql
	from dbutils.pooled_db import PooledDB
	
	dbname = '你的数据库名'
	dbhost = '你的数据库服务器地址'
	dbport = 3306
	dbuser = '你的数据库用户名'
	dbpwd = '你的数据库密码'
	
	config = {
	    'creator': pymysql,
	    'host': dbhost,
	    'port': dbport,
	    'user': dbuser,
	    'password': dbpwd,
	    'db': dbname,
	    'charset': "utf8",
	    'blocking': True,
	    'cursorclass': pymysql.cursors.DictCursor
	}
	
	_dbPool = PooledDB(**config)
	
	def get_db_connect():
	    return _dbPool.connection()

然后导入jxWebSQLGetDBConnection函数：

	from jxWebUI import jxWebSQLGetDBConnection

然后将get_db_connect设置给jxWebUI：

	jxWebSQLGetDBConnection(get_db_connect)

<font color=red size=3>注1：</font>不用jxWebSQLGetDBConnection设置数据库连接的获取，不影响jxWebUI的使用，只是不能使用数据表功能并在事件函数中操作数据库

<font color=red size=3>注2：</font>jxWebSQLGetDBConnection设置的数据库连接默认是PooledDB+myqsl组合，如果不是此组合，则请阅读后面的数据库连接接口说明

<font color=red size=3>注3：</font>上述配置由于未做初始化的数据库连接，可能会导致第一次查询时反应缓慢，而由于显示页面时会同步完成页面的disp事件，而所有的事件函数都需要一个数据库连接来完成数据库事务的处理，所以有可能会导致第一次打开页面会很慢

2、定义相应的数据库查询语句

使用capa的sql修饰符来定义一个select语句：

	@capa.sql
	def sqlTest(ci, db, ctx):
	    '''
	        select Name as devName, Type as devType, ID as devID from Device where Name like **devName:string** ;
	    '''
	    pass

<font color=red size=3>注1：</font>sql修饰的并非标准的select语句，不同点在where子句中多了一种变量查询格式

<font color=red size=3>注2：</font>sql修饰的查询语句只支持select、from、where和order by子句，支持多表联合查询，但不支持嵌套查询

<font color=red size=3>注3：</font>sql修饰的查询语句只支持and连接

变量查询的格式为：

	sqlAliasCol op=(Equal|NoEqual|Great|GreatEqual|Less|LessEqual|Like) VARIABLE Colon DATATYPE

	其中：
	sqlAliasCol：是【列名】或【表名.列名】
	op：是条件比较符，包括=、!=、>、>=、<、<=、like，分别代表等于、不等、大于、大于等于、小于、小于等于、类似【以什么开头的字符串】
	VARIABLE：是【查询条件表】中某个查询条件的输入变量所对应的bind的数据变量名
	DATATYPE：是该数据变量的数据类型，主要包括：int、float、bool、string、datatime
		1、bool一般都是用字节等来表示的
		2、string、datatime在查询时必须用引号框起来

当sql中定义了变量查询格式后，jxWebUI在响应reSearch事件时，会自动将【VARIABLE Colon DATATYPE】翻译成该数据变量的用户输入值，如上面的【devName:string】就会翻译成用户在设备名的输入栏中输入的值。

如用户在设备名中输入了：sh，则实际的查询语句为：

	SELECT Name AS devName, Type AS devType, ID AS devID FROM Device WHERE Name like 'sh%' LIMIT 15 OFFSET 0

即希望查询所有设备名以sh开头的设备。

如果用户未输入设备名，则该项查询条件会被忽略。上述的语句由于只有这一个查询条件，所以整个where子句全部被省略，相当于查询所有的设备：

	SELECT Name AS devName, Type AS devType, ID AS devID FROM Device LIMIT 15 OFFSET 0

用户点击分页标签，jxWebUI会自动更新该页码对应的OFFSET。

3、将定义的sql语句和数据查询页面进行关联

通过在将数据查询页面添加到快捷栏时指定dataSource属性就可实现二者的关联：

	capa.shortCutTree_add_item('测试', '测试数据库', 'test_web2', dataSource='sqlTest')

关联后，只要slelct子句中指定的各列名和数据表的列名相符，数据即会自动显示到数据表中。

## 数据库连接

前面说过：jxWebSQLGetDBConnection设置的数据库连接默认是PooledDB+myqsl组合，如果不是此组合，则需要自定义数据库连接。

jxWebSQLGetDBConnection的签名为：

	jxWebSQLGetDBConnection(get_db_connect, is_custom=False)
	当需要自定义数据连接时，指定is_custom=True

is_custom为True时，送入的get_db_connect函数应返回自己生成的一个数据库连接，其应遵循如下的鸭子类型：
	
	class DB:
	    def commit(self):
	        #提交数据库事务
	        pass
	
	    def rollback(self):
	        #回滚数据库事务
	        pass
	
	    def execute(self, sql:str):
	        #执行数据库查询语句
	        pass
	
	    def fetchone(self) -> dict:
	        #jxWebUI会在execute后调用，获取查询结果的单行数据
	        pass
	
	    def fetchall(self) -> list[dict]:
	        #jxWebUI会在execute后调用，获取查询结果的多行数据
	        pass
	
	    def trans2String(self, vt:str, v) -> str:
	        #将数据转换为字符串形式
	        #string、datetime类型需要添加引号
	        #bool类型转换为1、0[需确认自己的数据库设计]
	        pass
	
	    def __enter__(self):
	        #进入时打开数据库连接并开启数据库事务
	        pass
	
	    def __exit__(self, type, value, trace):
	        #离开时关闭数据库连接
	        pass

原则上，必须实现commit和rollback，否则web控件的点击响应函数中对数据库的操作就很难保证原子性，这是非常危险的。

而实现了commit和rollback，由于jxWebUI在调用事件函数后会执行commit，如果异常则执行rollback，